@@ -10,15 +10,15 @@
  under the License.
 
  Alternatively, the contents of this file may be used under the terms
- of the GNU Lesser General Public license version 2 or later (LGPL2+),
+ of the GNU Lesser General Public license version 2 or later (LGPLv2+),
  in which case the provisions of LGPL License are applicable instead of
  those above.
 
  For feedback and questions about my Files and Projects please mail me,
  Alexander Matthes (Ziz) , ziz_at_mailbox.org
 */
-#define AHA_VERSION "0.5"
-#define AHA_YEAR "2018"
+#define AHA_VERSION "0.5.1"
+#define AHA_YEAR "2020"
 #include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -182,6 +182,11 @@ struct Options {
 	int stylesheet;
 	char *title;
 	int word_wrap;
+	int no_xml;
+	char* lang;
+	char* css;
+	int ignore_cr;
+	char* bodystyle;
 };
 
 int divide (int dividend, int divisor){
@@ -198,7 +203,7 @@ void make_rgb (int color_id, char str_rgb[12]){
 	{
 		int index = color_id - 232;
 		int grey = index * 256 / 24;
-		sprintf(str_rgb, "%d,%d,%d", grey, grey, grey);
+		sprintf(str_rgb, "%02x%02x%02x", grey, grey, grey);
 		return;
 	}
 	int index_R = divide((color_id - 16), 36);
@@ -224,7 +229,7 @@ void make_rgb (int color_id, char str_rgb[12]){
 	} else {
 		rgb_B = 0;
 	}
-	sprintf(str_rgb, "%d,%d,%d", rgb_R, rgb_G, rgb_B);
+	sprintf(str_rgb, "%02x%02x%02x", rgb_R, rgb_G, rgb_B);
 }
 
 #define VERSION_PRINTF_MAKRO \
@@ -242,7 +247,11 @@ struct Options parseArgs(int argc, char* args[])
 		.no_header = 0,
 		.stylesheet = 0,
 		.title = NULL,
-		.word_wrap = 0
+		.word_wrap = 0,
+		.no_xml = 0,
+		.lang = NULL,
+		.ignore_cr = 0,
+		.bodystyle = NULL
 	};
 
 	//Searching Parameters
@@ -255,26 +264,53 @@ struct Options parseArgs(int argc, char* args[])
 			printf("use: \033[1maha\033[0m <\033[4moptions\033[0m> [\033[4m-f file\033[0m]\n");
 			printf("     \033[1maha\033[0m (\033[4m--help\033[0m|\033[4m-h\033[0m|\033[4m-?\033[0m)\n");
 			printf("\033[1maha\033[0m reads the Input from a file or stdin and writes HTML-Code to stdout\n");
-			printf("\033[4moptions\033[0m: --black,      -b: \033[1;30m\033[1;47mBlack\033[0m Background and \033[1;37mWhite\033[0m \"standard color\"\n");
-			printf("         --pink,       -p: \033[1;35mPink\033[0m Background\n");
-			printf("         --stylesheet, -s: Use a stylesheet instead of inline styles\n");
-			printf("         --iso X,    -i X: Uses ISO 8859-X instead of utf-8. X must be 1..16\n");
-			printf("         --title X,  -t X: Gives the html output the title \"X\" instead of\n");
-			printf("                           \"stdin\" or the filename\n");
-			printf("         --line-fix,   -l: Uses a fix for inputs using control sequences to\n");
-			printf("                           change the cursor position like htop. It's a hot fix,\n");
-			printf("                           it may not work with any program like htop. Example:\n");
-			printf("                           \033[1mecho\033[0m q | \033[1mhtop\033[0m | \033[1maha\033[0m -l > htop.htm\n");
-			printf("         --word-wrap,  -w: Wrap long lines in the html file. This works with\n");
-			printf("                           CSS3 supporting browsers as well as many older ones.\n");
-			printf("         --no-header,  -n: Don't include header into generated HTML,\n");
-			printf("                           useful for inclusion in full HTML files.\n");
-			printf("Example: \033[1maha\033[0m --help | \033[1maha\033[0m --black > aha-help.htm\n");
-			printf("         Writes this help text to the file aha-help.htm\n\n");
+			printf("\033[4mOptions\033[0m:\n");
+			printf("      --black,       -b: \033[1;30m\033[1;47mBlack\033[0m Background and \033[1;37mWhite\033[0m \"standard color\"\n");
+			printf("      --pink,        -p: \033[1;35mPink\033[0m Background\n");
+			printf("      --style X,   -y X: Set the style used in the <body> element\n");
+			printf("      --stylesheet,  -s: Use a stylesheet instead of inline styles\n");
+			printf("      --iso X,     -i X: Uses ISO 8859-X instead of utf-8. X must be 1..16\n");
+			printf("      --title X,   -t X: Gives the html output the title \"X\" instead of\n");
+			printf("                         \"stdin\" or the filename\n");
+			printf("      --lang X,    -L X: Uses the ISO 639-1 code X for the language\n");
+			printf("      --line-fix,    -l: Uses a fix for inputs using control sequences to\n");
+			printf("                         change the cursor position like htop. It's a hot fix,\n");
+			printf("                         it may not work with any program like htop. Example:\n");
+			printf("                         \033[1mecho\033[0m q | \033[1mhtop\033[0m | \033[1maha\033[0m -l > htop.htm\n");
+			printf("      --word-wrap,   -w: Wrap long lines in the html file. This works with\n");
+			printf("                         CSS3 supporting browsers as well as many older ones.\n");
+			printf("      --no-header,   -n: Don't include header into generated HTML,\n");
+			printf("                         useful for inclusion in full HTML files.\n");
+			printf("      --no-xml,      -x: Don't use doctype xml but html (may useful for old \n");
+			printf("                         browsers like IE)\n");
+			printf("      --css X,     -c X: Add css file X to the output. In fact just adds \n");
+			printf("                         <link rel=\"stylesheet\" href=\"X\" /> to the header.\n");
+			printf("      --ignore-cr,   -r: Ignore all carriage-returns (ASCII sign 13, \\r)\n");
+			printf("                         which may lead to double new lines in html.\n");
+			printf("\033[4mExamples\033[0m:\n");
+			printf("          Create an HTML file with a black background, a custom title and\n");
+			printf("          a larger font-size using \033[1maha\033[0m's help:\n");
+			printf("\n");
+			printf("            \033[1;3;33m$\033[0m \033[1maha\033[0m -h | \033[1maha\033[0m -b -t '\033[1maha\033[0m help' -y 'font-size:1.8em' > \033[1maha\033[0m-help.html\n");
+			printf("\n");
+			printf("          Create an HTML file with a white background using the output of \033[3mdiff\033[0m\n");
+			printf("\n");
+			printf("            \033[1;3;33m$\033[0m diff -u --color=always oldfile.c newfile.c | \033[1maha\033[0m > diff.html\n");
+			printf("\n");
+			printf("          Create an HTML file with a black background from the output of \033[3mhtop\033[0m.\n");
+			printf("          You have to use option -l due the other new-line-commands \033[3mhtop\033[0m uses:\n");
+			printf("\n");
+			printf("            \033[1;3;33m$\033[0m echo q | htop | \033[1maha\033[0m -b -l > htop.html\n");
+			printf("\n");
+			printf("          Create an HTML file from the output of this man page. \033[3mman\033[0m uses bold\n");
+			printf("          and underline formatting from \033[3mnroff\033[0m, which \033[3mul\033[0m converts to SGR:\n");
+			printf("\n");
+			printf("            \033[1;3;33m$\033[0m MAN_KEEP_FORMATTING=1 COLUMNS=80 man \033[1maha\033[0m | ul | \033[1maha\033[0m > man-\033[1maha\033[0m.html\n");
+			printf("\n");
 			printf("Copyleft \033[1;32mAlexander Matthes\033[0m aka \033[4mZiz\033[0m "AHA_YEAR"\n");
 			printf("         \033[5;36mziz@mailbox.org\033[0m\n");
 			printf("         \033[5;36mhttps://github.com/theZiz/aha\033[0m\n");
-			printf("This application is subject to the \033[1;34mMPL\033[0m or \033[1;34mLGPL\033[0m.\n");
+			printf("This application is subject to the \033[1;34mMPL1.1\033[0m or \033[1;34mLGPLv2+\033[0m.\n");
 			exit(EXIT_SUCCESS);
 		}
 		else
@@ -351,6 +387,45 @@ struct Options parseArgs(int argc, char* args[])
 			opts.filename=args[p];
 		}
 		else
+		if ((strcmp(args[p],"--no-xml")==0) || (strcmp(args[p],"-x")==0))
+			opts.no_xml=1;
+		else
+		if ((strcmp(args[p],"--lang")==0) || (strcmp(args[p],"-L")==0))
+		{
+			if (p+1>=argc)
+			{
+				fprintf(stderr,"No ISO lang code given!\n");
+				exit(EXIT_FAILURE);
+			}
+			opts.lang = args[p+1];
+			p++;
+		}
+		else
+		if ((strcmp(args[p],"--css")==0) || (strcmp(args[p],"-c")==0))
+		{
+			if (p+1>=argc)
+			{
+				fprintf(stderr,"No css file given!\n");
+				exit(EXIT_FAILURE);
+			}
+			opts.css = args[p+1];
+			p++;
+		}
+		else
+		if ((strcmp(args[p],"--ignore-cr")==0) || (strcmp(args[p],"-r")==0))
+			opts.ignore_cr=1;
+		else
+		if ((strcmp(args[p],"--style")==0) || (strcmp(args[p],"-y")==0))
+		{
+			if (p+1>=argc)
+			{
+				fprintf(stderr,"No style given!\n");
+				exit(EXIT_FAILURE);
+			}
+			opts.bodystyle=args[p+1];
+			p++;
+		}
+		else
 		{
 			fprintf(stderr,"Unknown parameter \"%s\"\n",args[p]);
 			exit(EXIT_FAILURE);
@@ -428,15 +503,34 @@ void printHeader(const struct Options *opts)
 	char encoding[16] = "UTF-8";
 	if(opts->iso>0) snprintf(encoding, sizeof(encoding), "ISO-8859-%i", opts->iso);
 
-	printf("<?xml version=\"1.0\" encoding=\"%s\" ?>\n<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n", encoding);
+	if (opts->no_xml)
+		printf("<!DOCTYPE html>\n");
+	else
+		printf("<?xml version=\"1.0\" encoding=\"%s\" ?>\n<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n", encoding);
 	printf("<!-- This file was created with the aha Ansi HTML Adapter. https://github.com/theZiz/aha -->\n");
-	printf("<html xmlns=\"http://www.w3.org/1999/xhtml\">\n");
-	printf("<head>\n<meta http-equiv=\"Content-Type\" content=\"application/xml+xhtml; charset=%s\" />\n", encoding);
-
+	if (opts->no_xml)
+	{
+		if (opts->lang)
+			printf("<html lang=\"%s\">\n",opts->lang);
+		else
+			printf("<html>\n");
+		printf("<head>\n<meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n", encoding);
+	}
+	else
+	{
+		if (opts->lang)
+			printf("<html xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"%s\">\n",opts->lang);
+		else
+			printf("<html xmlns=\"http://www.w3.org/1999/xhtml\">\n");
+		printf("<head>\n<meta http-equiv=\"Content-Type\" content=\"application/xml+xhtml; charset=%s\"/>\n", encoding);
+	}
 	printf("<title>");
 	printHtml(opts->title ? opts->title : opts->filename ? opts->filename : "stdin");
 	printf("</title>\n");
 
+	if (opts->css)
+		printf("<link rel=\"stylesheet\" href=\"%s\" />\n", opts->css );
+
 	int style_tag = 0;
 	if (opts->stylesheet)
 	{
@@ -529,12 +623,25 @@ void printHeader(const struct Options *opts)
 	}
 	else
 	{
-		switch (opts->colorscheme)
+		printf("<body");
+		if(opts->bodystyle || opts->colorscheme==SCHEME_BLACK || opts->colorscheme==SCHEME_PINK)
 		{
-			case SCHEME_BLACK: printf("<body style=\"color:white; background-color:black\">\n"); break;
-			case SCHEME_PINK: printf("<body style=\"background-color:pink\">\n");	break;
-			case SCHEME_WHITE: printf("<body>\n"); break;
+		    int styles=0;
+		    printf(" style=\"");
+		    if(opts->colorscheme==SCHEME_BLACK) {
+			++styles;
+			printf("color:white; background-color:black");
+		    } else if(opts->colorscheme==SCHEME_PINK) {
+			++styles;
+			printf("background-color:pink");
+		    }
+		    if(opts->bodystyle) {
+			if(styles) printf(";");
+			fputs(opts->bodystyle, stdout);
+		    }
+		    printf("\"");
 		}
+		printf(">\n");
 	}
 
 	printf("<pre>\n");
@@ -683,8 +790,7 @@ int main(int argc,char* args[])
 								case 35:
 								case 36:
 								case 37:
-								case 38:
-								case 39: // 3X - Set foreground color
+								case 38: // 3X - Set foreground color
 									{
 										int *dest = &(state.fc);
 										if (negative != 0)
@@ -739,7 +845,11 @@ int main(int argc,char* args[])
 										}
 									}
 									break;
-
+								case 39: // Set foreground color to default
+									state.fc_colormode = MODE_3BIT;
+									state.highlighted = 0;
+									state.fc = -1;
+									break;
 								case 40:
 								case 41:
 								case 42:
@@ -748,8 +858,7 @@ int main(int argc,char* args[])
 								case 45:
 								case 46:
 								case 47:
-								case 48:
-								case 49: // 4X - Set background color
+								case 48: // 4X - Set background color
 									{
 										int *dest = &(state.bc);
 										if (negative != 0)
@@ -804,6 +913,11 @@ int main(int argc,char* args[])
 										}
 									}
 									break;
+								case 49: // Set background color to default
+									state.bc_colormode = MODE_3BIT;
+									state.highlighted = 0;
+									state.bc = -1;
+									break;
 								case 90:
 								case 91:
 								case 92:
@@ -938,11 +1052,11 @@ int main(int argc,char* args[])
 								{
 									char rgb[12];
 									make_rgb(state.fc,rgb);
-									printf("color: rgb(%s);",rgb);
+									printf("color:#%s;",rgb);
 								}
 								break;
 							case MODE_24BIT:
-								printf("color: #%06x;",state.fc);
+								printf("color:#%06x;",state.fc);
 								break;
 						};
 						if (opts.stylesheet &&
@@ -963,11 +1077,11 @@ int main(int argc,char* args[])
 								{
 									char rgb[12];
 									make_rgb(state.bc,rgb);
-									printf("background-color: rgb(%s);",rgb);
+									printf("background-color:#%s;",rgb);
 								}
 								break;
 							case MODE_24BIT:
-								printf("background-color: #%06x;",state.bc);
+								printf("background-color:#%06x;",state.bc);
 								break;
 						};
 						printf("\">");
@@ -1004,6 +1118,10 @@ int main(int argc,char* args[])
 			momline++;
 			printf("\n");
 		}
+		else
+		if (c==13 && opts.ignore_cr)
+		{
+		}
 		else if (c!=8)
 		{
 			line++;
@@ -1059,7 +1177,7 @@ int main(int argc,char* args[])
 
 	// If current state is different than the default, there is a <span> open - close it
 	if (statesDiffer(&state, &default_state))
-		printf("</span>\n");
+		printf("</span>");
 
 	//Footer
 	if (opts.no_header == 0)
